@@ -12,9 +12,10 @@ class Perceptron:
 
 
     def fit(self, x_train, y_train, lr=0.01, max_epochs=50):
-        for i in range(len(y_train)):
-            if y_train[i] == 0:
-                y_train[i] = -1
+        y_train_new_class = [y_train[i] for i in range(len(y_train))]
+        for i in range(len(y_train_new_class)):
+            if y_train_new_class[i] == 0:
+                y_train_new_class[i] = -1
         for i in tqdm(range(max_epochs)):
             cnt = 0
             for j in range(len(x_train)):
@@ -29,12 +30,12 @@ class Perceptron:
                 else:
                     y = -1
 
-                if y_train[j] == y:
+                if y_train_new_class[j] == y:
                     cnt += 1
                 else:
                     for k in range(len(x_sample)):
-                        self.param[k] += lr * y_train[j] * x_sample[k]
-            if len(y_train) == cnt:
+                        self.param[k] += lr * y_train_new_class[j] * x_sample[k]
+            if len(y_train_new_class) == cnt:
                 print('所有训练样本被正确分类，提前终止')
                 break
 
@@ -92,6 +93,74 @@ class KNN:
             first_key = next(iter(sorted_judge_list))
             predict_list.append(first_key[0])
         return predict_list
+
+class NaiveBayes:
+    def __init__(self):
+        self.class_index = []
+        self.class_num = []
+        self.pri_prob = []
+        self.average = []
+        self.variance = []
+
+    def fit(self, x_train, y_train):
+
+        for i in range(len(y_train)):
+            if not y_train[i] in self.class_index:
+                self.class_index.append(y_train[i])
+                self.class_num.append(1)
+            else:
+                self.class_num[self.class_index.index(y_train[i])] += 1
+
+
+        for i in range(len(self.class_index)):
+            self.pri_prob.append(self.class_num[i] / len(y_train))  # 先验概率
+
+            sample_average = [0 for _ in range(len(x_train[0]))]
+
+            for j in range(len(x_train)):
+                if y_train[j] == self.class_index[i]:
+                    for k in range(len(sample_average)):
+                        sample_average[k] += x_train[j][k]
+            for k in range(len(sample_average)):
+                sample_average[k] /= self.class_num[i]
+            self.average.append(sample_average)
+
+
+
+        for i in range(len(self.class_index)):
+            sigma = [0 for _ in range(len(x_train[0]))]
+            for j in range(len(x_train)):
+                if y_train[j] == self.class_index[i]:
+                    for k in range(len(x_train[j])):
+                        sigma[k] += (x_train[j][k] - self.average[i][k]) ** 2
+            for j in range(len(sigma)):
+                sigma[j] /= self.class_num[i]
+            self.variance.append(sigma)
+
+    def predict(self, x_test):
+        predict = []
+        for i in range(len(x_test)):
+            predict_list = []
+            for j in range(len(self.class_index)):
+                prob = self.pri_prob[j]
+                for k in range(len(x_test[i])):
+                    prob *= 1 / math.sqrt(2 * math.pi * self.variance[j][k]) * np.exp(-(x_test[i][k] - self.average[j][k]) ** 2 / (2 * self.variance[j][k]))
+                predict_list.append(prob)
+
+            max_index = max(predict_list)
+            predict_index = predict_list.index(max_index)
+
+            predict_class = self.class_index[predict_index]
+
+            predict.append(predict_class)
+        return predict
+
+
+
+
+
+
+
 '''
 class Decision_Tree:
     def __init__(self, type='ID3'):
