@@ -46,10 +46,13 @@ class MLP:
 
 
         self.loss_values = []
+        self.valid_loss_values = []
         self.accuracy_list = []
+        self.valid_accuracy_list = []
 
 
-    def train(self, x_train, y_train, lr=0.01, epochs=100, batch_size=1, verbose_frequence=1):
+
+    def train(self, x_train, y_train, x_valid='NULL', y_valid='NULL', lr=0.01, epochs=100, batch_size=1, verbose_frequence=1):
         if len(x_train) % batch_size != 0:
             raise ValueError(f"batch_size需为训练集样本数量的约数")
         ac = getattr(activation_function, self.hidden_activation)
@@ -141,12 +144,52 @@ class MLP:
 
                     for j in range(self.hidden_node_num + 1):
                         self.output_weight[j] -= lr * output_gradient_sum[j] / len(x_batch_list)
-                if (_ + 1) % verbose_frequence == 0:
-                    print(f"第{_ + 1}代损失：{epoch_loss / len(x_train)}，准确率：{cnt / len(x_train) * 100}%")
                 self.loss_values.append(epoch_loss / len(x_train))
                 self.accuracy_list.append(cnt / len(x_train))
+                if isinstance(x_valid, list) and isinstance(y_valid, list):
+                    valid_epoch_loss = 0
+                    valid_cnt = 0
+                    for i in range(len(x_valid)):
+                        x_sample = [x_valid[i][index] for index in range(len(x_valid[i]))]
+                        x_sample.append(1)
+                        hidden_layer_output = [0 for num in range(self.hidden_node_num)]
+                        for j in range(self.hidden_node_num):
+                            for k in range(self.input_node_num + 1):
+                                hidden_layer_output[j] += self.hidden_weight[j][k] * x_sample[k]
+
+                        hidden_layer_ac_output = [0 for num in range(self.hidden_node_num)]
+                        for j in range(self.hidden_node_num):
+                            hidden_layer_ac_output[j] = ac(hidden_layer_output[j])
+
+                        hidden_layer_ac_output.append(1)
+
+                        output_layer_output = 0
+                        for j in range(self.hidden_node_num + 1):
+                            output_layer_output += self.output_weight[j] * hidden_layer_ac_output[j]
+
+                        predict = activation_function.sigmoid(output_layer_output)
+
+                        pred_result = 1 if predict >= 0.5 else 0
+                        if pred_result == y_valid[i]:
+                            valid_cnt += 1
+                        epsilon = 1e-10
+                        cross_entropy_loss = - (y_valid[i] * np.log(predict + epsilon) + (
+                                1 - y_valid[i]) * np.log(1 - predict + epsilon))
+
+                        valid_epoch_loss += cross_entropy_loss  # 累加损失
+                    self.valid_loss_values.append(valid_epoch_loss / len(x_valid))
+                    self.valid_accuracy_list.append(valid_cnt / len(x_valid))
+                if (_ + 1) % verbose_frequence == 0:
+                    if isinstance(x_valid, list) and isinstance(y_valid, list):
+                        print(f"第{_ + 1}代损失：{self.loss_values[-1]}，准确率：{self.accuracy_list[-1] * 100}%"
+                              f"，验证集损失：{self.valid_loss_values[-1]}，验证集准确率：{self.valid_accuracy_list[-1] * 100}%")
+                    else:
+                        print(f"第{_ + 1}代损失：{self.loss_values[-1]}，准确率：{self.accuracy_list[-1] * 100}%")
+
+
+
+            return self.loss_values, self.accuracy_list, self.valid_loss_values, self.valid_accuracy_list
 
-            return self.loss_values, self.accuracy_list
         if self.target == 'multi_classify':
             for _ in tqdm(range(epochs)):
 
@@ -252,12 +295,58 @@ class MLP:
                     for j in range(self.output_node_num):
                         for k in range(self.hidden_node_num + 1):
                             self.output_weight[j][k] -= lr * output_gradient_sum[j][k] / len(x_batch_list)
-                if (_ + 1) % verbose_frequence == 0:
-                    print(f"第{_ + 1}代损失：{epoch_loss / len(x_train)}，准确率：{cnt / len(x_train) * 100}%")
                 self.loss_values.append(epoch_loss / len(x_train))
                 self.accuracy_list.append(cnt / len(x_train))
+                if isinstance(x_valid, list) and isinstance(y_valid, list):
 
-            return self.loss_values, self.accuracy_list
+                    valid_epoch_loss = 0
+                    valid_cnt = 0
+                    for i in range(len(x_valid)):
+                        x_sample = [x_valid[i][index] for index in range(len(x_valid[i]))]
+                        x_sample.append(1)
+                        hidden_layer_output = [0 for num in range(self.hidden_node_num)]
+                        for j in range(self.hidden_node_num):
+                            for k in range(self.input_node_num + 1):
+                                hidden_layer_output[j] += self.hidden_weight[j][k] * x_sample[k]
+
+                        hidden_layer_ac_output = [0 for num in range(self.hidden_node_num)]
+                        for j in range(self.hidden_node_num):
+                            hidden_layer_ac_output[j] = ac(hidden_layer_output[j])
+
+                        hidden_layer_ac_output.append(1)
+
+                        output_layer_output = [0 for num in range(self.output_node_num)]
+                        for j in range(self.output_node_num):
+                            for k in range(self.hidden_node_num + 1):
+                                output_layer_output[j] += self.output_weight[j][k] * hidden_layer_ac_output[k]
+
+                        predict_list = []
+                        for j in range(self.output_node_num):
+                            predict = activation_function.softmax(j, output_layer_output)
+                            predict_list.append(predict)
+
+                        pred_result = [0 for num in range(self.output_node_num)]
+                        max_prob = max(predict_list)
+                        index = predict_list.index(max_prob)
+                        pred_result[index] = 1
+
+                        if pred_result == y_valid[i]:
+                            valid_cnt += 1
+                        epsilon = 1e-10
+                        categorical_cross_entropy_loss = 0
+                        for j in range(self.output_node_num):
+                            categorical_cross_entropy_loss += -y_valid[i][j] * np.log(predict_list[j])
+                        valid_epoch_loss += categorical_cross_entropy_loss  # 累加损失
+                    self.valid_loss_values.append(valid_epoch_loss / len(x_valid))
+                    self.valid_accuracy_list.append(valid_cnt / len(x_valid))
+                if (_ + 1) % verbose_frequence == 0:
+                    if isinstance(x_valid, list) and isinstance(y_valid, list):
+                        print(f"第{_ + 1}代损失：{self.loss_values[-1]}，准确率：{self.accuracy_list[-1] * 100}%"
+                              f"，验证集损失：{self.valid_loss_values[-1]}，验证集准确率：{self.valid_accuracy_list[-1] * 100}%")
+                    else:
+                        print(f"第{_ + 1}代损失：{self.loss_values[-1]}，准确率：{self.accuracy_list[-1] * 100}%")
+
+            return self.loss_values, self.accuracy_list, self.valid_loss_values, self.valid_accuracy_list
 
         if self.target == 'output_1_node_regress':
             for _ in tqdm(range(epochs)):
@@ -338,10 +427,42 @@ class MLP:
 
                     for j in range(self.hidden_node_num + 1):
                         self.output_weight[j] -= lr * output_gradient_sum[j] / len(x_batch_list)
-                if (_ + 1) % verbose_frequence == 0:
-                    print(f"第{_ + 1}代损失：{epoch_loss / len(x_train)}")
                 self.loss_values.append(epoch_loss / len(x_train))
 
+                if isinstance(x_valid, list) and isinstance(y_valid, list):
+                    valid_epoch_loss = 0
+                    for i in range(len(x_valid)):
+                        x_sample = [x_valid[i][index] for index in range(len(x_valid[i]))]
+                        x_sample.append(1)
+                        hidden_layer_output = [0 for num in range(self.hidden_node_num)]
+                        for j in range(self.hidden_node_num):
+                            for k in range(self.input_node_num + 1):
+                                hidden_layer_output[j] += self.hidden_weight[j][k] * x_sample[k]
+
+                        hidden_layer_ac_output = [0 for num in range(self.hidden_node_num)]
+                        for j in range(self.hidden_node_num):
+                            hidden_layer_ac_output[j] = ac(hidden_layer_output[j])
+
+                        hidden_layer_ac_output.append(1)
+
+                        output_layer_output = 0
+                        for j in range(self.hidden_node_num + 1):
+                            output_layer_output += self.output_weight[j] * hidden_layer_ac_output[j]
+
+                        mse = 0.5 * (output_layer_output - y_valid[i]) ** 2
+
+                        valid_epoch_loss += mse
+
+                    self.valid_loss_values.append(valid_epoch_loss / len(x_valid))
+
+                if (_ + 1) % verbose_frequence == 0:
+                    if isinstance(x_valid, list) and isinstance(y_valid, list):
+                        print(f"第{_ + 1}代损失：{self.loss_values[-1]}，验证集损失：{self.valid_loss_values[-1]}")
+                    else:
+                        print(f"第{_ + 1}代损失：{self.loss_values[-1]}")
+
+            return self.loss_values, self.valid_loss_values
+
         if self.target == 'output_multi_node_regress':
             for _ in tqdm(range(epochs)):
 
@@ -434,10 +555,45 @@ class MLP:
                     for j in range(self.output_node_num):
                         for k in range(self.hidden_node_num + 1):
                             self.output_weight[j][k] -= lr * output_gradient_sum[j][k] / len(x_batch_list)
-                if (_ + 1) % verbose_frequence == 0:
-                    print(f"第{_ + 1}代损失：{epoch_loss / len(x_train)}")
                 self.loss_values.append(epoch_loss / len(x_train))
 
+                if isinstance(x_valid, list) and isinstance(y_valid, list):
+                    valid_epoch_loss = 0
+                    for i in range(len(x_valid)):
+                        x_sample = [x_valid[i][index] for index in range(len(x_valid[i]))]
+                        x_sample.append(1)
+                        hidden_layer_output = [0 for num in range(self.hidden_node_num)]
+                        for j in range(self.hidden_node_num):
+                            for k in range(self.input_node_num + 1):
+                                hidden_layer_output[j] += self.hidden_weight[j][k] * x_sample[k]
+
+                        hidden_layer_ac_output = [0 for num in range(self.hidden_node_num)]
+                        for j in range(self.hidden_node_num):
+                            hidden_layer_ac_output[j] = ac(hidden_layer_output[j])
+
+                        hidden_layer_ac_output.append(1)
+
+                        output_layer_output = [0 for num in range(self.output_node_num)]
+                        for j in range(self.output_node_num):
+                            for k in range(self.hidden_node_num + 1):
+                                output_layer_output[j] += self.output_weight[j][k] * hidden_layer_ac_output[k]
+
+                        mse = 0
+                        for j in range(self.output_node_num):
+                            mse += 0.5 * (output_layer_output[j] - y_valid[i][j]) ** 2
+
+                        valid_epoch_loss += mse  # 累加损失
+
+                    self.valid_loss_values.append(valid_epoch_loss / len(x_valid))
+
+                if (_ + 1) % verbose_frequence == 0:
+                    if isinstance(x_valid, list) and isinstance(y_valid, list):
+                        print(f"第{_ + 1}代损失：{self.loss_values[-1]}，验证集损失：{self.valid_loss_values[-1]}")
+                    else:
+                        print(f"第{_ + 1}代损失：{self.loss_values[-1]}")
+
+            return self.loss_values, self.valid_loss_values
+
     def predict(self, x):
         if self.target == '2_classify':
             ac = getattr(activation_function, self.hidden_activation)
